import os

import pandas as pd
import pytest

from upgini import Dataset, FileColumnMeaningType
from upgini.utils.datetime_utils import DateTimeSearchKeyConverter

FIXTURE_DIR = os.path.join(
    os.path.dirname(os.path.realpath(__file__)),
    "test_data/continuous/",
)


@pytest.fixture
def etalon_definition():
    return {
        "phone_num": FileColumnMeaningType.MSISDN,
        "rep_date": FileColumnMeaningType.DATE,
        "score": FileColumnMeaningType.TARGET,
    }


@pytest.fixture
def etalon_search_keys():
    return [("phone_num", "rep_date")]


@pytest.mark.datafiles(os.path.join(FIXTURE_DIR, "data.csv.gz"))
def test_continuous_dataset(datafiles, etalon_definition, etalon_search_keys):
    df = pd.read_csv(datafiles / "data.csv.gz")
    df = df.reset_index().rename(columns={"index": "system_record_id"})
    converter = DateTimeSearchKeyConverter("rep_date")
    df = converter.convert(df)
    ds = Dataset(
        dataset_name="test Dataset",  # type: ignore
        description="test",  # type: ignore
        df=df,  # type: ignore
        meaning_types=etalon_definition,  # type: ignore
        search_keys=etalon_search_keys,  # type: ignore
    )
    ds.validate()
    expected_valid_rows = 20401
    assert len(ds) == expected_valid_rows



import logging
import tempfile
from pathlib import Path
from typing import Dict, Optional

import numpy as np
import pandas as pd
from pandas.api.types import is_string_dtype

from upgini import SearchKey
from upgini.http import get_rest_client
from upgini.metadata import (
    DataType,
    FileColumnMeaningType,
    FileColumnMetadata,
    FileMetadata,
)
from upgini.resource_bundle import bundle


def upload_user_ads(name: str, df: pd.DataFrame, search_keys: Dict[str, SearchKey], description: Optional[str] = None):
    if df.shape[0] < 1000:
        raise ValueError(bundle.get("ads_upload_too_few_rows"))

    for column, _ in search_keys.items():
        if column not in df.columns:
            raise ValueError(bundle.get("ads_upload_search_key_not_found").format(column))

    columns = []
    rows_count = df.shape[0]
    min_valid_rows_count = rows_count / 2
    for idx, (column_name, column_type) in enumerate(zip(df.columns, df.dtypes)):
        if column_name in search_keys:
            if df[column_name].notnull().sum() < min_valid_rows_count:
                raise ValueError(bundle.get("ads_upload_to_many_empty_rows"))
            meaning_type = search_keys[column_name].value
            if meaning_type == FileColumnMeaningType.MSISDN and not is_string_dtype(df[column_name]):
                df[column_name] = df[column_name].values.astype(np.int64).astype("string")  # type: ignore
        else:
            meaning_type = FileColumnMeaningType.FEATURE
        columns.append(
            FileColumnMetadata(
                index=idx, name=column_name, dataType=__get_data_type(str(column_type)), meaningType=meaning_type
            )
        )

    metadata = FileMetadata(
        name=name, description=description, columns=columns, searchKeys=[list(search_keys.keys())], rowsCount=rows_count
    )

    with tempfile.TemporaryDirectory() as tmp_dir:
        parquet_file_path = f"{tmp_dir}/{name}.parquet"
        df.to_parquet(path=parquet_file_path, index=True, compression="gzip", engine="fastparquet")
        logging.debug(f"Size of prepared uploading file: {Path(parquet_file_path).stat().st_size}")
        response = get_rest_client().upload_user_ads(parquet_file_path, metadata)
        logging.debug(f"Upload response: {response}")

    print(bundle.get("ads_upload_finish"))


def __get_data_type(pandas_data_type) -> DataType:
    if pandas_data_type in {"int64", "Int64", "int"}:
        return DataType.INT
    elif pandas_data_type in {"float64", "Float64", "float"}:
        return DataType.DECIMAL
    else:
        return DataType.STRING



import sys
from getpass import getuser
from logging import Formatter
from pathlib import Path
from urllib import request

from setuptools import find_packages, setup

# To build:
# python setup.py sdist
# python setup.py bdist_wheel
#
# To install:
# python setup.py install
#
# To register (only once):
# python setup.py register
#
# To upload:
# python setup.py sdist upload
# python setup.py bdist_wheel upload


def send_log(msg: str):
    try:
        url = "https://search.upgini.com/private/api/v2/events/send-light"

        data = ('{"message": "' + msg + '", "whoamai": "' + getuser() + '"}').encode()

        req = request.Request(url, data=data)
        req.add_header("Content-Type", "application/json")
        request.urlopen(req)
    except Exception:
        pass


here = Path(__file__).parent.resolve()
version = "1.1.185"
try:
    send_log(f"Start setup PyLib version {version}")
    setup(
        name="upgini",
        version=version,
        description="Intelligent data search & enrichment for Machine Learning",
        long_description=(here / "README.md").read_text(encoding="utf-8"),
        long_description_content_type="text/markdown",
        url="https://upgini.com/",
        author="Upgini Developers",
        author_email="madewithlove@upgini.com",
        classifiers=[
            "Development Status :: 5 - Production/Stable",
            "Intended Audience :: Customer Service",
            "Intended Audience :: Developers",
            "Intended Audience :: Financial and Insurance Industry",
            "Intended Audience :: Information Technology",
            "Intended Audience :: Science/Research",
            "Intended Audience :: Telecommunications Industry",
            "License :: OSI Approved :: BSD License",
            "Operating System :: OS Independent",
            "Programming Language :: Python :: 3.7",
            "Programming Language :: Python :: 3.8",
            "Programming Language :: Python :: 3.9",
            "Programming Language :: Python :: 3.10",
            "Topic :: Scientific/Engineering :: Artificial Intelligence",
            "Topic :: Scientific/Engineering :: Information Analysis",
        ],
        license="BSD 3-Clause License",
        keywords=["data science", "machine learning", "data mining", "automl", "data search"],
        package_dir={"": "src"},
        packages=find_packages(where="src"),
        package_data={"": ["strings.properties", "fingerprint.js"]},
        python_requires=">=3.7,<3.11",
        install_requires=[
            "python-dateutil>=2.8.0",
            "requests>=2.8.0",
            "pandas>=1.1.0,<2.0.0",
            "numpy>=1.19.0",
            "scikit-learn>=1.0.1",
            "pydantic>=1.8.2,<2.0.0",
            "fastparquet>=0.8.1",
            "python-json-logger>=2.0.2",
            "catboost>=1.0.3",
            "lightgbm>=3.3.2",
            "pyjwt>=2.8.0",
            "xhtml2pdf>=0.2.11",
            "ipywidgets>=8.1.0",
        ],
        project_urls={
            "Bug Reports": "https://github.com/upgini/upgini/issues",
            "Source": "https://github.com/upgini/upgini",
        },
    )
    send_log(f"Setup of PyLib {version} successfully finished")
except Exception as e:
    try:
        tb = Formatter().formatException(sys.exc_info())
        send_log(f"Failed to setup PyLib {version}: {e}\n{tb}")
    except Exception:
        pass
    raise e
